"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
const runner_1 = require("../utils/runner");
const languageModes_1 = require("../modes/languageModes");
const nullMode_1 = require("../modes/nullMode");
const documentService_1 = require("./documentService");
const vueInfoService_1 = require("./vueInfoService");
const dependencyService_1 = require("./dependencyService");
const _ = require("lodash");
class VLS {
    constructor(lspConnection) {
        this.lspConnection = lspConnection;
        this.clientDynamicRegisterSupport = false;
        this.formatterRegistration = null;
        this.pendingValidationRequests = {};
        this.validationDelayMs = 500;
        this.validation = {
            'vue-html': true,
            html: true,
            css: true,
            scss: true,
            less: true,
            postcss: true,
            javascript: true
        };
        this.documentService = new documentService_1.DocumentService(this.lspConnection);
        this.vueInfoService = new vueInfoService_1.VueInfoService();
        this.dependencyService = new dependencyService_1.DependencyService();
        this.languageModes = new languageModes_1.LanguageModes();
    }
    init(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const workspacePath = params.rootPath;
            if (!workspacePath) {
                console.error('No workspace path found. Vetur initialization failed.');
                return {
                    capabilities: {}
                };
            }
            this.clientDynamicRegisterSupport = this.getClientCapability(params, 'workspace.symbol.dynamicRegistration', false);
            this.workspacePath = workspacePath;
            yield this.vueInfoService.init(this.languageModes);
            yield this.dependencyService.init(workspacePath, _.get(params.initializationOptions.config, ['vetur', 'useWorkspaceDependencies'], false));
            yield this.languageModes.init(workspacePath, {
                infoService: this.vueInfoService,
                dependencyService: this.dependencyService
            });
            this.setupConfigListeners();
            this.setupLSPHandlers();
            this.setupFileChangeListeners();
            this.lspConnection.onShutdown(() => {
                this.dispose();
            });
            if (params.initializationOptions && params.initializationOptions.config) {
                this.configure(params.initializationOptions.config);
            }
        });
    }
    listen() {
        this.lspConnection.listen();
    }
    getClientCapability(params, name, def) {
        const keys = name.split('.');
        let c = params.capabilities;
        for (let i = 0; c && i < keys.length; i++) {
            if (!c.hasOwnProperty(keys[i])) {
                return def;
            }
            c = c[keys[i]];
        }
        return c;
    }
    setupConfigListeners() {
        this.lspConnection.onDidChangeConfiguration(({ settings }) => {
            this.configure(settings);
            if (this.clientDynamicRegisterSupport) {
                const enableFormatter = settings && settings.vue && settings.vue.format && settings.vue.format.enable;
                if (enableFormatter) {
                    if (!this.formatterRegistration) {
                        this.formatterRegistration = this.lspConnection.client.register(vscode_languageserver_1.DocumentFormattingRequest.type, { documentSelector: [{ language: 'vue' }] });
                    }
                }
                else if (this.formatterRegistration) {
                    this.formatterRegistration.then(r => r.dispose());
                    this.formatterRegistration = null;
                }
            }
        });
        this.documentService.getAllDocuments().forEach(this.triggerValidation);
    }
    setupLSPHandlers() {
        this.lspConnection.onCompletion(this.onCompletion.bind(this));
        this.lspConnection.onCompletionResolve(this.onCompletionResolve.bind(this));
        this.lspConnection.onDefinition(this.onDefinition.bind(this));
        this.lspConnection.onDocumentRangeFormatting(this.onDocumentFormatting.bind(this));
        this.lspConnection.onDocumentHighlight(this.onDocumentHighlight.bind(this));
        this.lspConnection.onDocumentLinks(this.onDocumentLinks.bind(this));
        this.lspConnection.onDocumentSymbol(this.onDocumentSymbol.bind(this));
        this.lspConnection.onHover(this.onHover.bind(this));
        this.lspConnection.onReferences(this.onReferences.bind(this));
        this.lspConnection.onSignatureHelp(this.onSignatureHelp.bind(this));
        this.lspConnection.onDocumentColor(this.onDocumentColors.bind(this));
        this.lspConnection.onColorPresentation(this.onColorPresentations.bind(this));
    }
    setupFileChangeListeners() {
        this.documentService.onDidChangeContent((change) => {
            this.triggerValidation(change.document);
        });
        this.documentService.onDidClose(e => {
            this.removeDocument(e.document);
            this.cleanPendingValidation(e.document);
            this.lspConnection.sendDiagnostics({ uri: e.document.uri, diagnostics: [] });
        });
        this.lspConnection.onDidChangeWatchedFiles(({ changes }) => {
            let hasChanges = false;
            const jsMode = this.languageModes.getMode('javascript');
            changes.forEach(c => {
                if (c.type === vscode_languageserver_1.FileChangeType.Changed) {
                    const fsPath = vscode_uri_1.default.parse(c.uri).fsPath;
                    jsMode.onDocumentChanged(fsPath);
                    hasChanges = true;
                }
            });
            if (hasChanges) {
                this.documentService.getAllDocuments().forEach(d => {
                    this.triggerValidation(d);
                });
            }
        });
    }
    configure(config) {
        const veturValidationOptions = config.vetur.validation;
        this.validation['vue-html'] = veturValidationOptions.template;
        this.validation.css = veturValidationOptions.style;
        this.validation.postcss = veturValidationOptions.style;
        this.validation.scss = veturValidationOptions.style;
        this.validation.less = veturValidationOptions.style;
        this.validation.javascript = veturValidationOptions.script;
        this.languageModes.getAllModes().forEach(m => {
            if (m.configure) {
                m.configure(config);
            }
        });
    }
    /**
     * Custom Notifications
     */
    displayInfoMessage(msg) {
        this.lspConnection.sendNotification('$/displayInfo', msg);
    }
    displayWarningMessage(msg) {
        this.lspConnection.sendNotification('$/displayWarning', msg);
    }
    displayErrorMessage(msg) {
        this.lspConnection.sendNotification('$/displayError', msg);
    }
    /**
     * Language Features
     */
    onDocumentFormatting({ textDocument, options }, token) {
        return runner_1.runSafeAsync(() => __awaiter(this, void 0, void 0, function* () {
            const doc = this.documentService.getDocument(textDocument.uri);
            const fullDocRange = vscode_languageserver_1.Range.create(vscode_languageserver_1.Position.create(0, 0), doc.positionAt(doc.getText().length));
            const modeRanges = this.languageModes.getModesInRange(doc, fullDocRange);
            const allEdits = [];
            const errMessages = [];
            modeRanges.forEach(range => {
                if (range.mode && range.mode.format) {
                    try {
                        const edits = range.mode.format(doc, range, options);
                        for (const edit of edits) {
                            allEdits.push(edit);
                        }
                    }
                    catch (err) {
                        errMessages.push(err.toString());
                    }
                }
            });
            if (errMessages.length !== 0) {
                this.displayErrorMessage('Formatting failed: "' + errMessages.join('\n') + '"');
                return [];
            }
            return allEdits;
        }), [], `Error while formatting range for ${textDocument.uri}`, token);
    }
    onCompletion({ textDocument, position }, token) {
        return runner_1.runSafeAsync(() => __awaiter(this, void 0, void 0, function* () {
            const doc = this.documentService.getDocument(textDocument.uri);
            const mode = this.languageModes.getModeAtPosition(doc, position);
            if (mode && mode.doComplete) {
                return mode.doComplete(doc, position);
            }
            return nullMode_1.NULL_COMPLETION;
        }), null, `Error while computing completions for ${textDocument.uri}`, token);
    }
    onCompletionResolve(item, token) {
        return runner_1.runSafeAsync(() => __awaiter(this, void 0, void 0, function* () {
            if (item.data) {
                const { uri, languageId } = item.data;
                if (uri && languageId) {
                    const doc = this.documentService.getDocument(uri);
                    const mode = this.languageModes.getMode(languageId);
                    if (doc && mode && mode.doResolve) {
                        return mode.doResolve(doc, item);
                    }
                }
            }
            return item;
        }), item, `Error while resolving completion proposal`, token);
    }
    onHover({ textDocument, position }, token) {
        return runner_1.runSafeAsync(() => __awaiter(this, void 0, void 0, function* () {
            const doc = this.documentService.getDocument(textDocument.uri);
            const mode = this.languageModes.getModeAtPosition(doc, position);
            if (mode && mode.doHover) {
                return mode.doHover(doc, position);
            }
            return nullMode_1.NULL_HOVER;
        }), null, `Error while computing hover for ${textDocument.uri}`, token);
    }
    onDocumentHighlight({ textDocument, position }) {
        const doc = this.documentService.getDocument(textDocument.uri);
        const mode = this.languageModes.getModeAtPosition(doc, position);
        if (mode && mode.findDocumentHighlight) {
            return mode.findDocumentHighlight(doc, position);
        }
        return [];
    }
    onDefinition({ textDocument, position }) {
        const doc = this.documentService.getDocument(textDocument.uri);
        const mode = this.languageModes.getModeAtPosition(doc, position);
        if (mode && mode.findDefinition) {
            return mode.findDefinition(doc, position);
        }
        return [];
    }
    onReferences({ textDocument, position }) {
        const doc = this.documentService.getDocument(textDocument.uri);
        const mode = this.languageModes.getModeAtPosition(doc, position);
        if (mode && mode.findReferences) {
            return mode.findReferences(doc, position);
        }
        return [];
    }
    onDocumentLinks({ textDocument }) {
        const doc = this.documentService.getDocument(textDocument.uri);
        const documentContext = {
            resolveReference: ref => {
                if (this.workspacePath && ref[0] === '/') {
                    return vscode_uri_1.default.file(path.resolve(this.workspacePath, ref)).toString();
                }
                const docUri = vscode_uri_1.default.parse(doc.uri);
                return docUri
                    .with({
                    // Reference from components need to go dwon from their parent dir
                    path: path.resolve(docUri.fsPath, '..', ref)
                })
                    .toString();
            }
        };
        const links = [];
        this.languageModes.getAllModesInDocument(doc).forEach(m => {
            if (m.findDocumentLinks) {
                pushAll(links, m.findDocumentLinks(doc, documentContext));
            }
        });
        return links;
    }
    onDocumentSymbol({ textDocument }, token) {
        return runner_1.runSafeAsync(() => __awaiter(this, void 0, void 0, function* () {
            const doc = this.documentService.getDocument(textDocument.uri);
            const symbols = [];
            this.languageModes.getAllModesInDocument(doc).forEach(m => {
                if (m.findDocumentSymbols) {
                    pushAll(symbols, m.findDocumentSymbols(doc));
                }
            });
            return symbols;
        }), [], `Error while computing document symbols for ${textDocument.uri}`, token);
    }
    onDocumentColors({ textDocument }, token) {
        return runner_1.runSafeAsync(() => __awaiter(this, void 0, void 0, function* () {
            const doc = this.documentService.getDocument(textDocument.uri);
            const colors = [];
            this.languageModes.getAllModesInDocument(doc).forEach(m => {
                if (m.findDocumentColors) {
                    pushAll(colors, m.findDocumentColors(doc));
                }
            });
            return colors;
        }), [], `Error while computing document colors for ${textDocument.uri}`, token);
    }
    onColorPresentations({ textDocument, color, range }, token) {
        return runner_1.runSafeAsync(() => __awaiter(this, void 0, void 0, function* () {
            const doc = this.documentService.getDocument(textDocument.uri);
            const mode = this.languageModes.getModeAtPosition(doc, range.start);
            if (mode && mode.getColorPresentations) {
                return mode.getColorPresentations(doc, color, range);
            }
            return [];
        }), [], `Error while computing color presentations for ${textDocument.uri}`, token);
    }
    onSignatureHelp({ textDocument, position }) {
        const doc = this.documentService.getDocument(textDocument.uri);
        const mode = this.languageModes.getModeAtPosition(doc, position);
        if (mode && mode.doSignatureHelp) {
            return mode.doSignatureHelp(doc, position);
        }
        return nullMode_1.NULL_SIGNATURE;
    }
    /**
     * Validations
     */
    triggerValidation(textDocument) {
        this.cleanPendingValidation(textDocument);
        this.pendingValidationRequests[textDocument.uri] = setTimeout(() => {
            delete this.pendingValidationRequests[textDocument.uri];
            this.validateTextDocument(textDocument);
        }, this.validationDelayMs);
    }
    cleanPendingValidation(textDocument) {
        const request = this.pendingValidationRequests[textDocument.uri];
        if (request) {
            clearTimeout(request);
            delete this.pendingValidationRequests[textDocument.uri];
        }
    }
    validateTextDocument(textDocument) {
        if (textDocument.getText().length === 0) {
            // ignore empty documents
            this.lspConnection.sendDiagnostics({ uri: textDocument.uri, diagnostics: [] });
            return;
        }
        const diagnostics = this.doValidate(textDocument);
        this.lspConnection.sendDiagnostics({ uri: textDocument.uri, diagnostics });
    }
    doValidate(doc) {
        const diagnostics = [];
        const self = this;
        if (doc.languageId === 'vue') {
            this.languageModes.getAllModesInDocument(doc).forEach(mode => {
                if (mode.doValidation && self.validation[mode.getId()]) {
                    pushAll(diagnostics, mode.doValidation(doc));
                }
            });
        }
        return diagnostics;
    }
    removeDocument(doc) {
        this.languageModes.onDocumentRemoved(doc);
    }
    dispose() {
        this.languageModes.dispose();
    }
    get capabilities() {
        return {
            textDocumentSync: vscode_languageserver_1.TextDocumentSyncKind.Full,
            completionProvider: { resolveProvider: true, triggerCharacters: ['.', ':', '<', '"', "'", '/', '@', '*'] },
            signatureHelpProvider: { triggerCharacters: ['('] },
            documentFormattingProvider: true,
            hoverProvider: true,
            documentHighlightProvider: true,
            documentLinkProvider: {
                resolveProvider: false
            },
            documentSymbolProvider: true,
            definitionProvider: true,
            referencesProvider: true,
            colorProvider: true
        };
    }
}
exports.VLS = VLS;
function pushAll(to, from) {
    if (from) {
        for (let i = 0; i < from.length; i++) {
            to.push(from[i]);
        }
    }
}
//# sourceMappingURL=vls.js.map